/**
 *  The Main Library of Betterville
 *  @author Drakkos
 *  @started 26th of March, 2009
 *
 */
 
#include "path.h"
#include <library.h>

// These are the defines for the two quests handled in this room.
#define   DOOR_QUEST        "betterville secret door"
#define   LIBRARY_QUEST     "betterville library"

inherit INHERITS + "inside_room";

// This mapping holds how many guesses a player has had to find the 
// secret panel.
mapping guesses;

string portrait_long();

void setup() {
  set_short ("second level of the library");
  add_property ("determinate", "the ");
  set_long ("This is the second level of the Betterville library.  It's full "
    "of portraits of crazy looking wizards.\n");
  set_light (100);  

  // Add in the add_item for the portraits.  
  add_item ("portrait", (: portrait_long() :));

  add_exit ("down", ROOMS + "main_library", "path");
}

void reset() {
  // Reset the mapping.
  guesses = ([ ]);
  // Call setup_data on the portrait handler.
  PORTRAIT_HANDLER->setup_data();
  // Send a message to the room indicating that the portraits have changed.
  tell_room (this_object(), "There is a strange flash, and all the "
    "portraits in the room morph and change into new and interesting "
    "shapes and combinations.\n");
}


void init() {
  ::init();
  // Add in the commands for the room.
  add_command ("look", "behind portrait of <string>");
  add_command ("enter", "code <string>");
}

int do_enter (object *indirect_obs, string dir_match, string indir_match, 
  mixed *args, string pattern) {
    // Get the code the player entered as a parameter.
    string entered_code = args[0];
    string needed_code;
    
//    if (!LIBRARY->query_quest_done (this_player()->query_name(), DOOR_QUEST)) {
//      add_failed_mess ("Enter what into what?  You're crazy, dude.\n", ({ }));
//      return 0;
//    }
    
    // Get the code associated with the player's successful completion of the 
    // 'betterville library' quest.  
    needed_code = LIBRARY->query_player_quest_info (this_player()->query_name(), 
      LIBRARY_QUEST);
      
    // Compare the code that was entered against the code that was needed.
    if (entered_code != needed_code) {
      // It didn't match, so give a message to that effect.
      add_succeeded_mess ("$N enter$s a code on a secret number pad, but "
        "nothing happens.\n", ({ }));
      return 1;
    }

    // It was the right code, so handle the succeeded message.
    add_succeeded_mess ("$N enter$s a code on a secret number pad.\n", 
      ({ }));
    // Handle the movingh of the player as a call_out to ensure that the
    // messages are shown in the right order.
    call_out ("transport_player", 0, this_player());
    return 1;
}

/**
 *
 *  This method transports a player to the third floor of the library.
 *
 *  @param        player        The player to teleport.
 *
 */
 
void transport_player (object player) {
  tell_object (player, "The world goes black for an instant, and when "
    "you open your eyes you find yourself somewhere new...\n");
  player->move_with_look (ROOMS + "library_third_level", 
    "$N appear$s with a flash!", "$N disappear$s with a flash of light!");    
}

int do_look (object *indirect_obs, string dir_match, string indir_match, 
  mixed *args, string pattern) {
  
  string wizard;
  int panel;
  int wiz;
  int num_guesses;
 
  // Get the name of the wizard they are looking for. 
  wizard = lower_case (args[0]);
  
  // Check to see if the wizard exists in the handler.
  wiz = PORTRAIT_HANDLER->find_portrait (wizard);
  
  if (wiz == -1) {
    // No wizard, so print out a failed mess.
    add_failed_mess ("You can't find a portrait of that wizard...\n", 
      ({ }));
    return 0;
  }
  
  // Get how many guesses the player has had this reset period.
  num_guesses = guesses[this_player()->query_name()];
  
  // Add one to it
  num_guesses += 1;
  
  // Store the number of guesses for the player.
  
  guesses[this_player()->query_name()] = num_guesses;
  
  // Check to see if they've guessed more than three times.
  if (num_guesses > 3) {
    // Sorry, you had your chance - wait until the next reset period.
    add_failed_mess ("None of the portraits will move for you!  How odd.\n", 
      ({ }));
    return 0;
  }
  
  // Let's give a message appropriate for the number of guesses the player
  // has made.
  switch (num_guesses) {
    case 1:
      tell_room (this_object(), "The painting moves easily, but there is "
        "an ominous glow that appears and then fades away...\n");
      break;
    case 2:
      tell_room (this_object(), "It is hard to move the portrait - as if "
        "someone was holding on from the other side of the frame.  Odd.\n");
      break;
    case 3:
      tell_room (this_object(), "It is almost impossible to move the "
        "portrait.  It's like it has suddenly become magically glued to "
        "the wall!\n");
      break;
  }       
  
  // Okay, get the secret panel value of the portrait being checked.
  panel = PORTRAIT_HANDLER->query_portrait_panel (wizard);  
  
  // If the result is 2, then it's the portrait we're looking for.
  if (panel == 2) {
    tell_object (this_player(), "There is a number-pad behind the portrait of "   
      + cap_words (wizard) + "!  It's just waiting for you to enter a number.\n");
    // Well done, you get the quest!
    LIBRARY->set_player_quest_info (this_player()->query_name(), DOOR_QUEST, 
      1);
  }
  
  // Show the succeeded_mess
  add_succeeded_mess ("$N move$s aside a picture and look$s behind it.\n", 
    ({ }));
  return 1;
}

/**
 *
 *  Get the long description of the portrait add_item
 *
 *  @returns            The add_item description
 *
 */
 
string portrait_long() {
  string *portraits = PORTRAIT_HANDLER->query_portraits();
 
  // This for loop turns the lower case array elements into 
  // appropriately capitalised elements. 
  for (int i = 0; i < sizeof (portraits); i++) {
    portraits[i] = cap_words (portraits[i]);
  }
  
  // Return the long description.
  return "There are many portraits hanginging around the room.  You can see "
    "portraits for the following wizards: " + query_multiple_short (portraits) 
    + ".  Undoubtedly more information about each of them could be researched "
    "in the library.\n";
}
