/** 
 *  The Beast of Betterville
 *  @author Drakkos
 *  @started 30/03/2009
 *
 */
 
#include "path.h"
// We need smart_fighter.h to get access to the defines for set_smart_fighter.
#include <smart_fighter.h>;

// Use the betterville_npc inherit.  We changed that in this chapter to 
// use the smart fighter inherit.
inherit INHERITS + "betterville_npc";

void setup() {
  set_name ("beast");
  set_short ("The Beast");
  add_adjective ("the");
  add_property ("determinate", "");
  set_long ("That the Beast was once man would never be apparent from his "
    "appearance.  The magic that worked through him has shed away every "
    "last trace of humanity, leaving only a core of pure animal rage.\n");

  add_adjective (({"lady", "tempesre"}));
  add_alias (({"lady", "tempesre"}));
  
  set_gender (1);
  basic_setup ("werewolf", "warrior", 500 + random (50));
  add_skill_level ("adventuring.health", 500);
  setup_nationality ("/std/nationality/genua", "Genua");

  // Set the beast to use the smart fighter code.  He'll fight with 
  // unarmed, defend with dodge and parry, and when a spell is cast 
  // he will run away.
  set_smart_fighter (USE_UNARMED, DEFEND_BALANCED, SPELL_REACT_RUN_AWAY);
  
  // Add a combat action here - 25% chance of occuring each round.  
  // The second parameter is an identifier for us if we later need 
  // to refer to the attack.  The third parameter is the function that 
  // is to be called when the action triggers.  In this case, it calls 
  // the function lick_wounds.
  add_combat_action (25, "lick wounds", ({"lick_wounds"}));
  
  load_chat(120,
  ({
    1, ": sits up on his haunches and mournfully grooms himself.",
    1, "@sigh sadly",
    1, ": stares sadly into space",
  }));


  load_a_chat(120,
  ({
    1, "#animal_growling",
    1, "@roar",
  }));

  add_respond_to_with( ({ "@say", 
       ({"lilith"}), }), 
      "#lilith_response");  

  add_respond_to_with( ({ "@say", 
       ({"beast"}), }), 
      "#beast_response"); 
}

/**
 *
 *  This method gives a randomised 'middle section' of a growl 
 *  consisting of aaaarraaa or something similiar.
 *
 *  @returns                A randomised growl bit
 *
 */
string random_growling_bits() {
  // Allocate creates an array.  The size of the array is set by the 
  // first parameter. The starting value of each element of the array 
  // is set by the second.  For this, we create an array of between 
  // 3 and 6 elements, each containing the letter 'a'.
  string *start_bit = allocate (3 + random (4), "a");
  // 3 and 6 elements, each containing the letter 'r'.  
  string *mid_bit = allocate (3 + random (4), "r");
  // 3 and 6 elements, each containing the letter 'a'.  
  string *end_bit = allocate (3 + random (4), "a");

  // The implode function takes an array such as ({"a", "a", "a"}) and /
  // turns it into a string.  The first parameter is the array to implode.
  // The second is what to separate each element with - so ({"a", "a", "a"})
  // becomes the string "aaa".  We do this for each of the arrays that we
  // created above.
  return implode (start_bit, "") + 
    implode (mid_bit, "") + implode (end_bit, "");

}

/**
 *  
 *  Make the beast emit an animalistic growl.
 *
 */
 
void animal_growling() {
  // These are the letters a growl can start with.
  string *start_letters = ({"Gra", "Ra", "Kra", "Bwa"});
  
  // Here we pick a random letter from the array above (gra, for example), 
  // then get a random growl (aarrraaa) and then add an 'r' to the 
  // end to make 'Graaarrraaar!'.  
  do_command ("say " + element_of (start_letters) 
    + random_growling_bits() + "r!");
  
}

/**
 *
 *  This method contorts a word.  It takes the first and last letters, 
 *  and puts growls between them.  The word 'hello' for example might 
 *  become 'haarraaro'.
 *
 *  @param        word        The string to contort
 *  @returns                  The contorted word
 *
 */
 
string contort_word (string word) {
  // Each element in a string is actually stored as an ASCII code.  
  // When we pull an element out of a string, we must store it in 
  // an int.
  int start = word[0];
  int end = word[<1];
 
  // To turn an integer ASCII code into a letter, we use sprintf.  
  // The %c code is used to parse the provided number as a letter.  
  // We do this for the first letter, add some random growling, 
  // and then append the last letter.
  return sprintf ("%c", start) + random_growling_bits() + sprintf ("%c", end);
}

/**
 *
 *  This method takes a string of text and randomly contorts words 
 *  in it.  
 *
 *  @param     str              The string to contort
 *  @returns                    The contorted string
 *
 */
 
string mangle_string (string str) {
  string *arr = explode (str, " ");
  
  for (int i = 0; i < sizeof (arr); i++) {
    if (!random (3)) {
      if (sizeof (arr[i])) {
        arr[i] = contort_word (arr[i]);
      }
    }
  }
  
  return implode (arr, " ");
}

/**
 *
 *  Response to the word 'lilith'
 *
 */
 
void lilith_response() {
  // Create the uncontorted response.
  string text = "Her!  A thousand curses upon her and her kin!  May "
    "the flesh be rent from her bones for her cruelty!  It was she who "
    "cursed me to this shape!";
    
  // When we say it, say it as a contorted string.
  init_command ("say " + mangle_string (text), 1);
}

/**
 *
 *  Response to the word 'beast'
 *
 */

void beast_response() {
  // Create the uncontorted response.
  string text = "Oh, fairytales are in the lifeblood of Genua.  Our land "
    "is forever changed by the power of raw and untamed narrative.  I am one "
    "of the victims of that legacy.";

  // When we say it, say it as a contorted string.
  init_command ("say " + mangle_string (text), 1);
}  

/**
 *
 *  This function is called by the combat code - it's the function 
 *  triggered when our 'lick wounds' attack occurs.
 *
 *  @param        attacker        The object doing the attacking
 *  @param        defender        The object doing the defending
 *
 */
 
void lick_wounds (object attacker, object defender) {
  // If we have less than 500 HPs...
  if (query_hp() < 500) {
    // Lick our wounds.
    init_command (": licks at his wounds for a moment, and seems to "
      "be healed a little from the process.", 1);
    // Adjust our HPs a little.
    adjust_hp (200);
  }
}